% Estimating the parameters of the mean-reversion process
% in order to simulate earnings

function [Sim_Earnings,MeanEarningsRate] = SimulateEarnings(T_vessel,nSim,...
    MR_earnings,year_earnings_MR,year_avg_mr,mr_avg_earnings,Plots)

St = log(MR_earnings);
dSt = diff(St);
dt = 1/52.177457; % one week/weeks per year
dStdt = dSt/dt;
St(end) = []; % to ensure vector of same length

% Polynomial curve fitting to estimate parameters
p = polyfit(St, dStdt, 1);
res = dStdt - polyval(p,St);  % evaluates the polynomial p at each point in St

kappa = -p(1);
meanReverting = p(2)/kappa;
sigma = std(res) * sqrt(dt);

%https://se.mathworks.com/help/finance/hwv.html
% hwv with constant volatility equals Ornstein-Uhlenbeck
OUmodel = hwv(kappa, meanReverting, sigma,  'StartState', St(end));

% Simulation 
n_years_sim = T_vessel;% 20 years of weekly simulation
n_Steps = round((1/dt)*n_years_sim); 
[Sim_Earnings, Time] = simulate(OUmodel,n_Steps,'nTrials',nSim, 'DeltaTime',dt);
Sim_Earnings = exp(Sim_Earnings); % Earnings again, not log earnings
Sim_Earnings = squeeze(Sim_Earnings);% Removal of redundant dimension
Sim_Earnings = Sim_Earnings';
Time = Time';

% Visualize Simulation Paths

[r,c] = size(Sim_Earnings);
Time_0 = 2020 + 107/366; % when earnings.xlsx data end (17 april 2020)
Time = Time_0 + Time;

if Plots == 2
    for i=1:r
        hold on
        plot(Time,Sim_Earnings(i,:));
    end
end

MeanInSim = zeros(1,length(nSim));
MedianInSim = zeros(1,length(nSim));
skew = zeros(1,length(nSim));
for i=1:nSim
    MeanInSim(i) = mean(Sim_Earnings(i,:));
    MedianInSim(i) = median(Sim_Earnings(i,:));
    skew(i) = skewness(Sim_Earnings(i,:));
end

% Find when year change and takes mean earnings of each year
YearChanging = zeros(1, T_vessel);
for i= 1:T_vessel
    j=1;
    yearcheck = 2020+i;
    while Time(j) < yearcheck && j < length(Time)
        j = j + 1;  
    end
    YearChanging(i) = (j-1);
end
                              
SimsPerYear = zeros(1,(length(YearChanging)+1)); % +1 to incl until mid april
SimsPerYear(1) = YearChanging(1);

for i=2:length(YearChanging)
    SimsPerYear(i) = YearChanging(i) - YearChanging(i-1);
end
% further to include from 2020+T_vessel until mid april
mid_april = c - sum(SimsPerYear);           
SimsPerYear(end) = mid_april;

MeanEarningsRate = zeros(r,length(SimsPerYear));
AccumulatedSimEarnings = zeros(r,length(SimsPerYear));
yearly = zeros(r,length(SimsPerYear));
for q=1:r 
    for n = 1:length(YearChanging)
        j = 1;
        accumSimEarnings = 0;
        while j <= YearChanging(n) && j <= length(Sim_Earnings)
            accumSimEarnings = Sim_Earnings(q,j) + accumSimEarnings;
            j = j + 1;       
        end
        AccumulatedSimEarnings(q,n) = accumSimEarnings;  
    end
    yearly(q,1) = AccumulatedSimEarnings(q,1);
    for i=2:length(SimsPerYear)
        yearly(q,i) = AccumulatedSimEarnings(q,i) - AccumulatedSimEarnings(q,i-1);   
    end
    % and again adding for mid april (Last column of yearly)
    midapril = sum(Sim_Earnings(q,:)) - AccumulatedSimEarnings(q,T_vessel);
    yearly(q,end) = midapril;
    for i=1:length(SimsPerYear)
        MeanEarningsRate(q,i) = yearly(q,i)/SimsPerYear(i);
    end
end %q

%preparing for plot
if Plots

%     figure
%     histfit(MeanInSim,50,'Lognormal')
%     pHat = lognfit(MeanInSim);
%     title('Average earnings for each simulation')
%     skewness_future_earnings = skewness(MeanInSim);
%     kurtosis_future_earnings = kurtosis(MeanInSim);

    x = 2020:(2020 + T_vessel);
    x_val = zeros(1,2*length(x));
    x_val(1:2:end) = x(1:1:end);
    x_val(2:2:end) = x(1:1:end);
    x_val(2) = Time_0;
    x_val(1) = [];
    x_val(end+1) = T_vessel + Time_0;
    y_val = zeros(1,2*length(MeanEarningsRate));
    year_sim_end = year_earnings_MR(end) + (n_Steps*dt);
    year_sim = year_earnings_MR(end):dt:year_sim_end;
    
    figure
    histogram(Sim_Earnings,100)
    xlim([0 6e4])
    xlabel('Simulated earnings [USD/day]')
    ylabel('Frequency')
    onedim = Sim_Earnings(:);
    kurt = kurtosis(onedim);
    skew = skewness(onedim);
    prc10 = prctile(onedim,10);
    prc90 = prctile(onedim,90);
    mdn = median(onedim);
    avg = mean(onedim);
    line([mdn, mdn], ylim, 'LineWidth', 1.5, 'Color', 'r');
    line([avg, avg], ylim,'Color', 'g','LineWidth', 1.5);
    line([prc10, prc10], ylim,'Color', 'black','LineWidth', 1.5);
    line([prc90, prc90], ylim,'Color', 'black','LineWidth', 1.5);
    
    figure(99)
    histogram(Sim_Earnings(1,:),50)
    x=Sim_Earnings(1,:);
    prc10 = prctile(x,10);
    prc90 = prctile(x,90);
    mdn = median(x);
    avg = mean(x);
    line([mdn, mdn], ylim, 'LineWidth', 1.5, 'Color', 'r');
    line([avg, avg], ylim,'Color', 'g','LineWidth', 1.5);
    line([prc10, prc10], ylim,'Color', 'black','LineWidth', 1.5);
    line([prc90, prc90], ylim,'Color', 'black','LineWidth', 1.5);
    xlim([0 35e3])
    ylim([0 80])
    xlabel('Earnings [USD/day]')
    ylabel('Frequency')
    x0=10;
    y0=10;
    width=400;
    height=300;
    set(gcf,'position',[x0,y0,width,height]);
    figure(100)
    histogram(Sim_Earnings(2,:),50)
    x=Sim_Earnings(2,:);
    prc10 = prctile(x,10);
    prc90 = prctile(x,90);
    mdn = median(x);
    avg = mean(x);
    line([mdn, mdn], ylim, 'LineWidth', 1.5, 'Color', 'r');
    line([avg, avg], ylim,'Color', 'g','LineWidth', 1.5);
    line([prc10, prc10], ylim,'Color', 'black','LineWidth', 1.5);
    line([prc90, prc90], ylim,'Color', 'black','LineWidth', 1.5);
    xlim([0 35e3])
    ylim([0 80])
    xlabel('Earnings [USD/day]')
    ylabel('Frequency')
    x0=10;
    y0=10;
    width=400;
    height=300;
    set(gcf,'position',[x0,y0,width,height]);

   
    
    if nSim < 10
    for i=1:r
        y_val(1:2:end) = MeanEarningsRate(i,1:1:end);
        y_val(2:2:end) = MeanEarningsRate(i,1:1:end);
        figure(i+1)
        hold on
        plot(year_earnings_MR,MR_earnings,'color','cyan','LineWidth',1);
        plot(year_avg_mr,mr_avg_earnings,'-.','color','black','LineWidth',2);
        plot(year_sim,Sim_Earnings(i,:),'r','LineWidth',1);
        plot(x_val,y_val,'-.','color','black','LineWidth',2);
        legend('Historical earnings','Annual average','Simulated earnings');
        xlabel('Year')
        ylabel('Earnings [USD/day]')
        xlim([1989.99 2040.2])
        ylim([0 55000])
            if i > 1
                break
            end
        hold off
    end
    
end
end
